@@ -227,58 +227,62 @@ impl CExp<i64> for ColumnI64 {
         util::getu64(data, self.off) as i64
     }
 }
-pub struct ColumnI32 {
+
+pub struct ColumnI {
     pub off: usize,
+    pub size: usize,
 }
-impl CExp<i64> for ColumnI32 {
+impl CExp<i64> for ColumnI {
     fn eval(&self, _e: &mut EvalEnv, data: &[u8]) -> i64 {
-        util::get(data, self.off, 4) as i32 as i64
-    }
-}
-pub struct ColumnI16 {
-    pub off: usize,
-}
-impl CExp<i64> for ColumnI16 {
-    fn eval(&self, _e: &mut EvalEnv, data: &[u8]) -> i64 {
-        util::get(data, self.off, 2) as i16 as i64
+        util::iget(data, self.off, self.size)
     }
 }
+
 pub struct ColumnI8 {
     pub off: usize,
 }
+
 impl CExp<i64> for ColumnI8 {
     fn eval(&self, _e: &mut EvalEnv, data: &[u8]) -> i64 {
         data[self.off] as i8 as i64
     }
 }
+
 pub struct ColumnF64 {
     pub off: usize,
 }
+
 impl CExp<f64> for ColumnF64 {
     fn eval(&self, _e: &mut EvalEnv, data: &[u8]) -> f64 {
         util::getf64(data, self.off) as f64
     }
 }
+
 pub struct ColumnF32 {
     pub off: usize,
 }
+
 impl CExp<f64> for ColumnF32 {
     fn eval(&self, _e: &mut EvalEnv, data: &[u8]) -> f64 {
         util::getf32(data, self.off) as f64
     }
 }
+
 pub struct ColumnBool {
     pub off: usize,
 }
+
 impl CExp<bool> for ColumnBool {
     fn eval(&self, _e: &mut EvalEnv, data: &[u8]) -> bool {
         data[self.off] & 1 != 0
     }
 }
+
 pub struct ColumnString {
     pub off: usize,
     pub size: usize,
 }
+
 impl CExp<Value> for ColumnString {
     fn eval(&self, ee: &mut EvalEnv, data: &[u8]) -> Value {
         let bytes = get_bytes(&ee.db, &data[self.off..], self.size).0;
@@ -286,19 +290,23 @@ impl CExp<Value> for ColumnString {
         Value::String(Rc::new(str))
     }
 }
+
 pub struct ColumnBinary {
     pub off: usize,
     pub size: usize,
 }
+
 impl CExp<Value> for ColumnBinary {
     fn eval(&self, ee: &mut EvalEnv, data: &[u8]) -> Value {
         let bytes = get_bytes(&ee.db, &data[self.off..], self.size).0;
         Value::RcBinary(Rc::new(bytes))
     }
 }
+
 pub struct Local {
     pub num: usize,
 }
+
 impl CExp<f64> for Local {
     fn eval(&self, e: &mut EvalEnv, _d: &[u8]) -> f64 {
         if let Value::Float(v) = e.stack[e.bp + self.num] {
