@@ -342,9 +342,6 @@ impl<'a> Parser<'a> {
             b"int" => INT,
             b"string" => STRING,
             b"binary" => BINARY,
-            b"tinyint" => TINYINT,
-            b"smallint" => SMALLINT,
-            b"bigint" => BIGINT,
             b"float" => FLOAT,
             b"double" => DOUBLE,
             b"bool" => BOOL,
